_I='default'
_H=' / '
_G='...'
_F='nargs'
_E='-'
_D='_'
_C=True
_B=False
_A=None
import errno,inspect,os,sys
from contextlib import contextmanager
from functools import update_wrapper
from itertools import repeat
from ._unicodefun import _verify_python_env
from .exceptions import Abort
from .exceptions import BadParameter
from .exceptions import ClickException
from .exceptions import Exit
from .exceptions import MissingParameter
from .exceptions import UsageError
from .formatting import HelpFormatter
from .formatting import join_options
from .globals import pop_context
from .globals import push_context
from .parser import OptionParser
from .parser import split_opt
from .termui import confirm
from .termui import prompt
from .termui import style
from .types import BOOL
from .types import convert_type
from .types import IntRange
from .utils import echo
from .utils import make_default_short_help
from .utils import make_str
from .utils import PacifyFlushWrapper
_missing=object()
SUBCOMMAND_METAVAR='COMMAND [ARGS]...'
SUBCOMMANDS_METAVAR='COMMAND1 [ARGS]... [COMMAND2 [ARGS]...]...'
DEPRECATED_HELP_NOTICE=' (DEPRECATED)'
DEPRECATED_INVOKE_NOTICE='DeprecationWarning: The command {name} is deprecated.'
def _maybe_show_deprecated_notice(cmd):
	if cmd.deprecated:echo(style(DEPRECATED_INVOKE_NOTICE.format(name=cmd.name),fg='red'),err=_C)
def fast_exit(code):sys.stdout.flush();sys.stderr.flush();os._exit(code)
def _bashcomplete(cmd,prog_name,complete_var=_A):
	B=prog_name;A=complete_var
	if A is _A:A=f"_{B}_COMPLETE".replace(_E,_D).upper()
	C=os.environ.get(A)
	if not C:return
	from ._bashcomplete import bashcomplete as D
	if D(cmd,B,A,C):fast_exit(1)
def _check_multicommand(base_command,cmd_name,cmd,register=_B):
	B=cmd_name;A=base_command
	if not A.chain or not isinstance(cmd,MultiCommand):return
	if register:C='It is not possible to add multi commands as children to another multi command that is in chain mode.'
	else:C='Found a multi command as subcommand to a multi command that is in chain mode. This is not supported.'
	raise RuntimeError(f"{C}. Command {A.name!r} is set to chain and {B!r} was added as a subcommand but it in itself is a multi command. ({B!r} is a {type(cmd).__name__} within a chained {type(A).__name__} named {A.name!r}).")
def batch(iterable,batch_size):return list(zip(*repeat(iter(iterable),batch_size)))
@contextmanager
def augment_usage_errors(ctx,param=_A):
	B=param
	try:yield
	except BadParameter as A:
		if A.ctx is _A:A.ctx=ctx
		if B is not _A and A.param is _A:A.param=B
		raise
	except UsageError as A:
		if A.ctx is _A:A.ctx=ctx
		raise
def iter_params_for_processing(invocation_order,declaration_order):
	def A(item):
		try:A=invocation_order.index(item)
		except ValueError:A=float('inf')
		return not item.is_eager,A
	return sorted(declaration_order,key=A)
class ParameterSource:
	COMMANDLINE='COMMANDLINE';ENVIRONMENT='ENVIRONMENT';DEFAULT='DEFAULT';DEFAULT_MAP='DEFAULT_MAP';VALUES={COMMANDLINE,ENVIRONMENT,DEFAULT,DEFAULT_MAP}
	@classmethod
	def validate(A,value):
		B=value
		if B not in A.VALUES:raise ValueError(f"Invalid ParameterSource value: {B!r}. Valid values are: {','.join(A.VALUES)}")
class Context:
	def __init__(A,command,parent=_A,info_name=_A,obj=_A,auto_envvar_prefix=_A,default_map=_A,terminal_width=_A,max_content_width=_A,resilient_parsing=_B,allow_extra_args=_A,allow_interspersed_args=_A,ignore_unknown_options=_A,help_option_names=_A,token_normalize_func=_A,color=_A,show_default=_A):
		O=info_name;N=color;M=token_normalize_func;L=ignore_unknown_options;K=allow_interspersed_args;J=allow_extra_args;I=max_content_width;H=terminal_width;G=default_map;F=obj;E=help_option_names;D=command;C=auto_envvar_prefix;B=parent;A.parent=B;A.command=D;A.info_name=O;A.params={};A.args=[];A.protected_args=[]
		if F is _A and B is not _A:F=B.obj
		A.obj=F;A._meta=getattr(B,'meta',{})
		if G is _A and B is not _A and B.default_map is not _A:G=B.default_map.get(O)
		A.default_map=G;A.invoked_subcommand=_A
		if H is _A and B is not _A:H=B.terminal_width
		A.terminal_width=H
		if I is _A and B is not _A:I=B.max_content_width
		A.max_content_width=I
		if J is _A:J=D.allow_extra_args
		A.allow_extra_args=J
		if K is _A:K=D.allow_interspersed_args
		A.allow_interspersed_args=K
		if L is _A:L=D.ignore_unknown_options
		A.ignore_unknown_options=L
		if E is _A:
			if B is not _A:E=B.help_option_names
			else:E=['--help']
		A.help_option_names=E
		if M is _A and B is not _A:M=B.token_normalize_func
		A.token_normalize_func=M;A.resilient_parsing=resilient_parsing
		if C is _A:
			if B is not _A and B.auto_envvar_prefix is not _A and A.info_name is not _A:C=f"{B.auto_envvar_prefix}_{A.info_name.upper()}"
		else:C=C.upper()
		if C is not _A:C=C.replace(_E,_D)
		A.auto_envvar_prefix=C
		if N is _A and B is not _A:N=B.color
		A.color=N;A.show_default=show_default;A._close_callbacks=[];A._depth=0;A._source_by_paramname={}
	def __enter__(A):A._depth+=1;push_context(A);return A
	def __exit__(A,exc_type,exc_value,tb):
		A._depth-=1
		if A._depth==0:A.close()
		pop_context()
	@contextmanager
	def scope(self,cleanup=_C):
		B=cleanup;A=self
		if not B:A._depth+=1
		try:
			with A as C:yield C
		finally:
			if not B:A._depth-=1
	@property
	def meta(self):return self._meta
	def make_formatter(A):return HelpFormatter(width=A.terminal_width,max_width=A.max_content_width)
	def call_on_close(A,f):A._close_callbacks.append(f);return f
	def close(A):
		for B in A._close_callbacks:B()
		A._close_callbacks=[]
	@property
	def command_path(self):
		A=self;B=''
		if A.info_name is not _A:B=A.info_name
		if A.parent is not _A:B=f"{A.parent.command_path} {B}"
		return B.lstrip()
	def find_root(B):
		A=B
		while A.parent is not _A:A=A.parent
		return A
	def find_object(B,object_type):
		A=B
		while A is not _A:
			if isinstance(A.obj,object_type):return A.obj
			A=A.parent
	def ensure_object(B,object_type):
		C=object_type;A=B.find_object(C)
		if A is _A:B.obj=A=C()
		return A
	def lookup_default(B,name):
		if B.default_map is not _A:
			A=B.default_map.get(name)
			if callable(A):A=A()
			return A
	def fail(A,message):raise UsageError(message,A)
	def abort(A):raise Abort()
	def exit(A,code=0):raise Exit(code)
	def get_usage(A):return A.command.get_usage(A)
	def get_help(A):return A.command.get_help(A)
	def invoke(*B,**E):
		F,A=B[:2];G=F
		if isinstance(A,Command):
			C=A;A=C.callback;G=Context(C,info_name=C.name,parent=F)
			if A is _A:raise TypeError('The given command does not have a callback that can be invoked.')
			for D in C.params:
				if D.name not in E and D.expose_value:E[D.name]=D.get_default(G)
		B=B[2:]
		with augment_usage_errors(F):
			with G:return A(*B,**E)
	def forward(*E,**A):
		B,D=E[:2]
		if not isinstance(D,Command):raise TypeError('Callback is not a command.')
		for C in B.params:
			if C not in A:A[C]=B.params[C]
		return B.invoke(D,**A)
	def set_parameter_source(B,name,source):A=source;ParameterSource.validate(A);B._source_by_paramname[name]=A
	def get_parameter_source(A,name):return A._source_by_paramname[name]
class BaseCommand:
	allow_extra_args=_B;allow_interspersed_args=_C;ignore_unknown_options=_B
	def __init__(B,name,context_settings=_A):
		A=context_settings;B.name=name
		if A is _A:A={}
		B.context_settings=A
	def __repr__(A):return f"<{A.__class__.__name__} {A.name}>"
	def get_usage(A,ctx):raise NotImplementedError('Base commands cannot get usage')
	def get_help(A,ctx):raise NotImplementedError('Base commands cannot get help')
	def make_context(A,info_name,args,parent=_A,**B):
		for (D,E) in A.context_settings.items():
			if D not in B:B[D]=E
		C=Context(A,info_name=info_name,parent=parent,**B)
		with C.scope(cleanup=_B):A.parse_args(C,args)
		return C
	def parse_args(A,ctx,args):raise NotImplementedError('Base commands do not know how to parse arguments.')
	def invoke(A,ctx):raise NotImplementedError('Base commands are not invokable by default')
	def main(E,args=_A,prog_name=_A,complete_var=_A,standalone_mode=_C,**G):
		D=standalone_mode;C=prog_name;B=args;_verify_python_env()
		if B is _A:B=sys.argv[1:]
		else:B=list(B)
		if C is _A:C=make_str(os.path.basename(sys.argv[0]if sys.argv else __file__))
		_bashcomplete(E,C,complete_var)
		try:
			try:
				with E.make_context(C,B,**G)as F:
					H=E.invoke(F)
					if not D:return H
					F.exit()
			except (EOFError,KeyboardInterrupt):echo(file=sys.stderr);raise Abort()
			except ClickException as A:
				if not D:raise
				A.show();sys.exit(A.exit_code)
			except OSError as A:
				if A.errno==errno.EPIPE:sys.stdout=PacifyFlushWrapper(sys.stdout);sys.stderr=PacifyFlushWrapper(sys.stderr);sys.exit(1)
				else:raise
		except Exit as A:
			if D:sys.exit(A.exit_code)
			else:return A.exit_code
		except Abort:
			if not D:raise
			echo('Aborted!',file=sys.stderr);sys.exit(1)
	def __call__(A,*B,**C):return A.main(*B,**C)
class Command(BaseCommand):
	def __init__(A,name,context_settings=_A,callback=_A,params=_A,help=_A,epilog=_A,short_help=_A,options_metavar='[OPTIONS]',add_help_option=_C,no_args_is_help=_B,hidden=_B,deprecated=_B):
		B='\x0c';BaseCommand.__init__(A,name,context_settings);A.callback=callback;A.params=params or[]
		if help and B in help:help=help.split(B,1)[0]
		A.help=help;A.epilog=epilog;A.options_metavar=options_metavar;A.short_help=short_help;A.add_help_option=add_help_option;A.no_args_is_help=no_args_is_help;A.hidden=hidden;A.deprecated=deprecated
	def __repr__(A):return f"<{A.__class__.__name__} {A.name}>"
	def get_usage(B,ctx):A=ctx.make_formatter();B.format_usage(ctx,A);return A.getvalue().rstrip('\n')
	def get_params(B,ctx):
		A=B.params;C=B.get_help_option(ctx)
		if C is not _A:A=A+[C]
		return A
	def format_usage(A,ctx,formatter):B=A.collect_usage_pieces(ctx);formatter.write_usage(ctx.command_path,' '.join(B))
	def collect_usage_pieces(A,ctx):
		B=[A.options_metavar]
		for C in A.get_params(ctx):B.extend(C.get_usage_pieces(ctx))
		return B
	def get_help_option_names(C,ctx):
		A=set(ctx.help_option_names)
		for B in C.params:A.difference_update(B.opts);A.difference_update(B.secondary_opts)
		return A
	def get_help_option(A,ctx):
		B=A.get_help_option_names(ctx)
		if not B or not A.add_help_option:return
		def C(ctx,param,value):
			A=ctx
			if value and not A.resilient_parsing:echo(A.get_help(),color=A.color);A.exit()
		return Option(B,is_flag=_C,is_eager=_C,expose_value=_B,callback=C,help='Show this message and exit.')
	def make_parser(C,ctx):
		A=ctx;B=OptionParser(A)
		for D in C.get_params(A):D.add_to_parser(B,A)
		return B
	def get_help(B,ctx):A=ctx.make_formatter();B.format_help(ctx,A);return A.getvalue().rstrip('\n')
	def get_short_help_str(A,limit=45):return A.short_help or A.help and make_default_short_help(A.help,limit)or''
	def format_help(A,ctx,formatter):C=formatter;B=ctx;A.format_usage(B,C);A.format_help_text(B,C);A.format_options(B,C);A.format_epilog(B,C)
	def format_help_text(B,ctx,formatter):
		A=formatter
		if B.help:
			A.write_paragraph()
			with A.indentation():
				C=B.help
				if B.deprecated:C+=DEPRECATED_HELP_NOTICE
				A.write_text(C)
		elif B.deprecated:
			A.write_paragraph()
			with A.indentation():A.write_text(DEPRECATED_HELP_NOTICE)
	def format_options(D,ctx,formatter):
		B=formatter;A=[]
		for E in D.get_params(ctx):
			C=E.get_help_record(ctx)
			if C is not _A:A.append(C)
		if A:
			with B.section('Options'):B.write_dl(A)
	def format_epilog(B,ctx,formatter):
		A=formatter
		if B.epilog:
			A.write_paragraph()
			with A.indentation():A.write_text(B.epilog)
	def parse_args(C,ctx,args):
		B=args;A=ctx
		if not B and C.no_args_is_help and not A.resilient_parsing:echo(A.get_help(),color=A.color);A.exit()
		D=C.make_parser(A);E,B,F=D.parse_args(args=B)
		for G in iter_params_for_processing(F,C.get_params(A)):H,B=G.handle_parse_result(A,E,B)
		if B and not A.allow_extra_args and not A.resilient_parsing:A.fail(f"Got unexpected extra argument{'s'if len(B)!=1 else''} ({' '.join(map(make_str,B))})")
		A.args=B;return B
	def invoke(A,ctx):
		_maybe_show_deprecated_notice(A)
		if A.callback is not _A:return ctx.invoke(A.callback,**ctx.params)
class MultiCommand(Command):
	allow_extra_args=_C;allow_interspersed_args=_B
	def __init__(A,name=_A,invoke_without_command=_B,no_args_is_help=_A,subcommand_metavar=_A,chain=_B,result_callback=_A,**G):
		E=chain;D=invoke_without_command;C=no_args_is_help;B=subcommand_metavar;Command.__init__(A,name,**G)
		if C is _A:C=not D
		A.no_args_is_help=C;A.invoke_without_command=D
		if B is _A:
			if E:B=SUBCOMMANDS_METAVAR
			else:B=SUBCOMMAND_METAVAR
		A.subcommand_metavar=B;A.chain=E;A.result_callback=result_callback
		if A.chain:
			for F in A.params:
				if isinstance(F,Argument)and not F.required:raise RuntimeError('Multi commands in chain mode cannot have optional arguments.')
	def collect_usage_pieces(A,ctx):B=Command.collect_usage_pieces(A,ctx);B.append(A.subcommand_metavar);return B
	def format_options(A,ctx,formatter):B=formatter;Command.format_options(A,ctx,B);A.format_commands(ctx,B)
	def resultcallback(A,replace=_B):
		def B(f):
			B=A.result_callback
			if B is _A or replace:A.result_callback=f;return f
			def C(__value,*A,**C):return f(B(__value,*A,**C),*A,**C)
			A.result_callback=D=update_wrapper(C,f);return D
		return B
	def format_commands(F,ctx,formatter):
		D=formatter;B=[]
		for C in F.list_commands(ctx):
			A=F.get_command(ctx,C)
			if A is _A:continue
			if A.hidden:continue
			B.append((C,A))
		if len(B):
			G=D.width-6-max((len(A[0])for A in B));E=[]
			for (C,A) in B:help=A.get_short_help_str(G);E.append((C,help))
			if E:
				with D.section('Commands'):D.write_dl(E)
	def parse_args(C,ctx,args):
		A=ctx
		if not args and C.no_args_is_help and not A.resilient_parsing:echo(A.get_help(),color=A.color);A.exit()
		B=Command.parse_args(C,A,args)
		if C.chain:A.protected_args=B;A.args=[]
		elif B:A.protected_args,A.args=B[:1],B[1:]
		return A.args
	def invoke(B,ctx):
		A=ctx
		def F(value):
			C=value
			if B.result_callback is not _A:C=A.invoke(B.result_callback,C,**A.params)
			return C
		if not A.protected_args:
			if B.invoke_without_command:
				if not B.chain:return Command.invoke(B,A)
				with A:Command.invoke(B,A);return F([])
			A.fail('Missing command.')
		D=A.protected_args+A.args;A.args=[];A.protected_args=[]
		if not B.chain:
			with A:
				E,G,D=B.resolve_command(A,D);A.invoked_subcommand=E;Command.invoke(B,A);C=G.make_context(E,D,parent=A)
				with C:return F(C.command.invoke(C))
		with A:
			A.invoked_subcommand='*'if D else _A;Command.invoke(B,A);H=[]
			while D:E,G,D=B.resolve_command(A,D);C=G.make_context(E,D,parent=A,allow_extra_args=_C,allow_interspersed_args=_B);H.append(C);D,C.args=C.args,[]
			I=[]
			for C in H:
				with C:I.append(C.command.invoke(C))
			return F(I)
	def resolve_command(D,ctx,args):
		A=ctx;B=make_str(args[0]);E=B;C=D.get_command(A,B)
		if C is _A and A.token_normalize_func is not _A:B=A.token_normalize_func(B);C=D.get_command(A,B)
		if C is _A and not A.resilient_parsing:
			if split_opt(B)[0]:D.parse_args(A,A.args)
			A.fail(f"No such command '{E}'.")
		return B,C,args[1:]
	def get_command(A,ctx,cmd_name):raise NotImplementedError()
	def list_commands(A,ctx):return[]
class Group(MultiCommand):
	def __init__(A,name=_A,commands=_A,**B):MultiCommand.__init__(A,name,**B);A.commands=commands or{}
	def add_command(C,cmd,name=_A):
		B=cmd;A=name;A=A or B.name
		if A is _A:raise TypeError('Command has no name.')
		_check_multicommand(C,A,B,register=_C);C.commands[A]=B
	def command(B,*C,**D):
		from .decorators import command as E
		def A(f):A=E(*C,**D)(f);B.add_command(A);return A
		return A
	def group(B,*C,**D):
		from .decorators import group
		def A(f):A=group(*C,**D)(f);B.add_command(A);return A
		return A
	def get_command(A,ctx,cmd_name):return A.commands.get(cmd_name)
	def list_commands(A,ctx):return sorted(A.commands)
class CommandCollection(MultiCommand):
	def __init__(A,name=_A,sources=_A,**B):MultiCommand.__init__(A,name,**B);A.sources=sources or[]
	def add_source(A,multi_cmd):A.sources.append(multi_cmd)
	def get_command(A,ctx,cmd_name):
		C=cmd_name
		for D in A.sources:
			B=D.get_command(ctx,C)
			if B is not _A:
				if A.chain:_check_multicommand(A,C,B)
				return B
	def list_commands(B,ctx):
		A=set()
		for C in B.sources:A.update(C.list_commands(ctx))
		return sorted(A)
class Parameter:
	param_type_name='parameter'
	def __init__(A,param_decls=_A,type=_A,required=_B,default=_A,callback=_A,nargs=_A,metavar=_A,expose_value=_C,is_eager=_B,envvar=_A,autocompletion=_A):
		D=expose_value;C=default;B=nargs;A.name,A.opts,A.secondary_opts=A._parse_decls(param_decls or(),D);A.type=convert_type(type,C)
		if B is _A:
			if A.type.is_composite:B=A.type.arity
			else:B=1
		A.required=required;A.callback=callback;A.nargs=B;A.multiple=_B;A.expose_value=D;A.default=C;A.is_eager=is_eager;A.metavar=metavar;A.envvar=envvar;A.autocompletion=autocompletion
	def __repr__(A):return f"<{A.__class__.__name__} {A.name}>"
	@property
	def human_readable_name(self):return self.name
	def make_metavar(A):
		if A.metavar is not _A:return A.metavar
		B=A.type.get_metavar(A)
		if B is _A:B=A.type.name.upper()
		if A.nargs!=1:B+=_G
		return B
	def get_default(A,ctx):
		if callable(A.default):B=A.default()
		else:B=A.default
		return A.type_cast_value(ctx,B)
	def add_to_parser(A,parser,ctx):0
	def consume_value(B,ctx,opts):
		C=ctx;A=opts.get(B.name);D=ParameterSource.COMMANDLINE
		if A is _A:A=B.value_from_envvar(C);D=ParameterSource.ENVIRONMENT
		if A is _A:A=C.lookup_default(B.name);D=ParameterSource.DEFAULT_MAP
		if A is not _A:C.set_parameter_source(B.name,D)
		return A
	def type_cast_value(A,ctx,value):
		C=value;B=ctx
		if A.type.is_composite:
			if A.nargs<=1:raise TypeError(f"Attempted to invoke composite type but nargs has been set to {A.nargs}. This is not supported; nargs needs to be set to a fixed value > 1.")
			if A.multiple:return tuple((A.type(D or(),A,B)for D in C or()))
			return A.type(C or(),A,B)
		def D(value,level):
			E=level;C=value
			if E==0:return A.type(C,A,B)
			return tuple((D(A,E-1)for A in C or()))
		return D(C,(A.nargs!=1)+bool(A.multiple))
	def process_value(B,ctx,value):
		A=value
		if A is not _A:return B.type_cast_value(ctx,A)
	def value_is_missing(A,value):
		B=value
		if B is _A:return _C
		if(A.nargs!=1 or A.multiple)and B==():return _C
		return _B
	def full_process_value(B,ctx,value):
		C=ctx;A=value;A=B.process_value(C,A)
		if A is _A and not C.resilient_parsing:
			A=B.get_default(C)
			if A is not _A:C.set_parameter_source(B.name,ParameterSource.DEFAULT)
		if B.required and B.value_is_missing(A):raise MissingParameter(ctx=C,param=B)
		return A
	def resolve_envvar_value(B,ctx):
		if B.envvar is _A:return
		if isinstance(B.envvar,(tuple,list)):
			for C in B.envvar:
				A=os.environ.get(C)
				if A is not _A:return A
		else:
			A=os.environ.get(B.envvar)
			if A!='':return A
	def value_from_envvar(B,ctx):
		A=B.resolve_envvar_value(ctx)
		if A is not _A and B.nargs!=1:A=B.type.split_envvar_value(A)
		return A
	def handle_parse_result(A,ctx,opts,args):
		B=ctx
		with augment_usage_errors(B,param=A):
			C=A.consume_value(B,opts)
			try:C=A.full_process_value(B,C)
			except Exception:
				if not B.resilient_parsing:raise
				C=_A
			if A.callback is not _A:
				try:C=A.callback(B,A,C)
				except Exception:
					if not B.resilient_parsing:raise
		if A.expose_value:B.params[A.name]=C
		return C,args
	def get_help_record(A,ctx):0
	def get_usage_pieces(A,ctx):return[]
	def get_error_hint(A,ctx):B=A.opts or[A.human_readable_name];return _H.join((repr(A)for A in B))
class Option(Parameter):
	param_type_name='option'
	def __init__(A,param_decls=_A,show_default=_B,prompt=_B,confirmation_prompt=_B,hide_input=_B,is_flag=_A,flag_value=_A,multiple=_B,count=_B,allow_from_autoenv=_C,type=_A,help=_A,hidden=_B,show_choices=_C,show_envvar=_B,**G):
		F=count;D=prompt;C=flag_value;B=is_flag;H=G.get(_I,_missing)is _missing;Parameter.__init__(A,param_decls,type=type,**G)
		if D is _C:E=A.name.replace(_D,' ').capitalize()
		elif D is _B:E=_A
		else:E=D
		A.prompt=E;A.confirmation_prompt=confirmation_prompt;A.hide_input=hide_input;A.hidden=hidden
		if B is _A:
			if C is not _A:B=_C
			else:B=bool(A.secondary_opts)
		if B and H:A.default=_B
		if C is _A:C=not A.default
		A.is_flag=B;A.flag_value=C
		if A.is_flag and isinstance(A.flag_value,bool)and type in[_A,bool]:A.type=BOOL;A.is_bool_flag=_C
		else:A.is_bool_flag=_B
		A.count=F
		if F:
			if type is _A:A.type=IntRange(min=0)
			if H:A.default=0
		A.multiple=multiple;A.allow_from_autoenv=allow_from_autoenv;A.help=help;A.show_default=show_default;A.show_choices=show_choices;A.show_envvar=show_envvar
		if __debug__:
			if A.nargs<0:raise TypeError('Options cannot have nargs < 0')
			if A.prompt and A.is_flag and not A.is_bool_flag:raise TypeError('Cannot prompt for flags that are not bools.')
			if not A.is_bool_flag and A.secondary_opts:raise TypeError('Got secondary option for non boolean flag.')
			if A.is_bool_flag and A.hide_input and A.prompt is not _A:raise TypeError('Hidden input does not work with boolean flag prompts.')
			if A.count:
				if A.multiple:raise TypeError('Options cannot be multiple and count at the same time.')
				elif A.is_flag:raise TypeError('Options cannot be count and flags at the same time.')
	def _parse_decls(J,decls,expose_value):
		I='/';C=[];F=[];A=_A;D=[]
		for B in decls:
			if B.isidentifier():
				if A is not _A:raise TypeError('Name defined twice')
				A=B
			else:
				H=';'if B[:1]==I else I
				if H in B:
					E,G=B.split(H,1);E=E.rstrip()
					if E:D.append(split_opt(E));C.append(E)
					G=G.lstrip()
					if G:F.append(G.lstrip())
				else:D.append(split_opt(B));C.append(B)
		if A is _A and D:
			D.sort(key=lambda x:-len(x[0]));A=D[0][1].replace(_E,_D).lower()
			if not A.isidentifier():A=_A
		if A is _A:
			if not expose_value:return _A,C,F
			raise TypeError('Could not determine name for option')
		if not C and not F:raise TypeError(f"No options defined but a name was passed ({A}). Did you mean to declare an argument instead of an option?")
		return A,C,F
	def add_to_parser(A,parser,ctx):
		C=parser;B={'dest':A.name,_F:A.nargs,'obj':A}
		if A.multiple:D='append'
		elif A.count:D='count'
		else:D='store'
		if A.is_flag:
			B.pop(_F,_A);E=f"{D}_const"
			if A.is_bool_flag and A.secondary_opts:C.add_option(A.opts,action=E,const=_C,**B);C.add_option(A.secondary_opts,action=E,const=_B,**B)
			else:C.add_option(A.opts,action=E,const=A.flag_value,**B)
		else:B['action']=D;C.add_option(A.opts,**B)
	def get_help_record(A,ctx):
		K=', ';E=ctx
		if A.hidden:return
		F=[]
		def G(opts):
			B,C=join_options(opts)
			if C:F[:]=[_C]
			if not A.is_flag and not A.count:B+=f" {A.make_metavar()}"
			return B
		H=[G(A.opts)]
		if A.secondary_opts:H.append(G(A.secondary_opts))
		help=A.help or'';C=[]
		if A.show_envvar:
			B=A.envvar
			if B is _A:
				if A.allow_from_autoenv and E.auto_envvar_prefix is not _A:B=f"{E.auto_envvar_prefix}_{A.name.upper()}"
			if B is not _A:J=K.join((str(A)for A in B))if isinstance(B,(list,tuple))else B;C.append(f"env var: {J}")
		if A.default is not _A and(A.show_default or E.show_default):
			if isinstance(A.show_default,str):D=f"({A.show_default})"
			elif isinstance(A.default,(list,tuple)):D=K.join((str(B)for B in A.default))
			elif inspect.isfunction(A.default):D='(dynamic)'
			else:D=A.default
			C.append(f"default: {D}")
		if A.required:C.append('required')
		if C:I=';'.join(C);help=f"{help}  [{I}]"if help else f"[{I}]"
		return ('; 'if F else _H).join(H),help
	def get_default(A,ctx):
		if A.is_flag and not A.is_bool_flag:
			for B in ctx.command.params:
				if B.name==A.name and B.default:return B.flag_value
			return _A
		return Parameter.get_default(A,ctx)
	def prompt_for_value(A,ctx):
		B=A.get_default(ctx)
		if A.is_bool_flag:return confirm(A.prompt,B)
		return prompt(A.prompt,default=B,type=A.type,hide_input=A.hide_input,show_choices=A.show_choices,confirmation_prompt=A.confirmation_prompt,value_proc=lambda x:A.process_value(ctx,x))
	def resolve_envvar_value(A,ctx):
		B=ctx;C=Parameter.resolve_envvar_value(A,B)
		if C is not _A:return C
		if A.allow_from_autoenv and B.auto_envvar_prefix is not _A:D=f"{B.auto_envvar_prefix}_{A.name.upper()}";return os.environ.get(D)
	def value_from_envvar(A,ctx):
		B=A.resolve_envvar_value(ctx)
		if B is _A:return _A
		C=(A.nargs!=1)+bool(A.multiple)
		if C>0 and B is not _A:
			B=A.type.split_envvar_value(B)
			if A.multiple and A.nargs!=1:B=batch(B,A.nargs)
		return B
	def full_process_value(A,ctx,value):
		C=value;B=ctx
		if C is _A and A.prompt is not _A and not B.resilient_parsing:return A.prompt_for_value(B)
		return Parameter.full_process_value(A,B,C)
class Argument(Parameter):
	param_type_name='argument'
	def __init__(B,param_decls,required=_A,**C):
		A=required
		if A is _A:
			if C.get(_I)is not _A:A=_B
			else:A=C.get(_F,1)>0
		Parameter.__init__(B,param_decls,required=A,**C)
		if B.default is not _A and B.nargs<0:raise TypeError('nargs=-1 in combination with a default value is not supported.')
	@property
	def human_readable_name(self):
		A=self
		if A.metavar is not _A:return A.metavar
		return A.name.upper()
	def make_metavar(A):
		if A.metavar is not _A:return A.metavar
		B=A.type.get_metavar(A)
		if not B:B=A.name.upper()
		if not A.required:B=f"[{B}]"
		if A.nargs!=1:B+=_G
		return B
	def _parse_decls(D,decls,expose_value):
		A=decls
		if not A:
			if not expose_value:return _A,[],[]
			raise TypeError('Could not determine name for argument')
		if len(A)==1:B=C=A[0];B=B.replace(_E,_D).lower()
		else:raise TypeError(f"Arguments take exactly one parameter declaration, got {len(A)}.")
		return B,[C],[]
	def get_usage_pieces(A,ctx):return[A.make_metavar()]
	def get_error_hint(A,ctx):return repr(A.make_metavar())
	def add_to_parser(A,parser,ctx):parser.add_argument(dest=A.name,nargs=A.nargs,obj=A)


_A=None
from threading import local
_local=local()
def get_current_context(silent=False):
	try:return _local.stack[-1]
	except (AttributeError,IndexError):
		if not silent:raise RuntimeError('There is no active click context.')
def push_context(ctx):_local.__dict__.setdefault('stack',[]).append(ctx)
def pop_context():_local.stack.pop()
def resolve_color_default(color=_A):
	A=color
	if A is not _A:return A
	B=get_current_context(silent=True)
	if B is not _A:return B.color


from datetime import tzinfo,timedelta
class TomlTz(tzinfo):
	def __init__(A,toml_offset):
		B=toml_offset
		if B=='Z':A._raw_offset='+00:00'
		else:A._raw_offset=B
		A._sign=-1 if A._raw_offset[0]=='-'else 1;A._hours=int(A._raw_offset[1:3]);A._minutes=int(A._raw_offset[4:6])
	def tzname(A,dt):return'UTC'+A._raw_offset
	def utcoffset(A,dt):return A._sign*timedelta(hours=A._hours,minutes=A._minutes)
	def dst(A,dt):return timedelta(0)


from __future__ import print_function,absolute_import,division
_e='tag:yaml.org,2002:'
_d='state'
_c='args'
_b='tag:yaml.org,2002:python/object:'
_a='__getstate__'
_Z='tag:yaml.org,2002:set'
_Y='-.inf'
_X='.inf'
_W='.nan'
_V='base64'
_U='utf-8'
_T='null'
_S='typ'
_R='tag:yaml.org,2002:python/object/new:'
_Q='tag:yaml.org,2002:timestamp'
_P='tag:yaml.org,2002:map'
_O='tag:yaml.org,2002:seq'
_N='tag:yaml.org,2002:float'
_M='tag:yaml.org,2002:binary'
_L='tag:yaml.org,2002:null'
_K=0.0
_J='|'
_I='%s.%s'
_H='.'
_G='tag:yaml.org,2002:int'
_F='comment'
_E='ascii'
_D='tag:yaml.org,2002:str'
_C=False
_B=True
_A=None
from .error import *
from .nodes import *
from .compat import text_type,binary_type,to_unicode,PY2,PY3
from .compat import ordereddict
from .compat import nprint,nprintf
from .scalarstring import LiteralScalarString,FoldedScalarString,SingleQuotedScalarString,DoubleQuotedScalarString,PlainScalarString
from .scalarint import ScalarInt,BinaryInt,OctalInt,HexInt,HexCapsInt
from .scalarfloat import ScalarFloat
from .scalarbool import ScalarBoolean
from .timestamp import TimeStamp
import datetime,sys,types
if PY3:import copyreg,base64
else:import copy_reg as copyreg
if _C:from typing import Dict,List,Any,Union,Text,Optional
__all__=['BaseRepresenter','SafeRepresenter','Representer','RepresenterError','RoundTripRepresenter']
class RepresenterError(YAMLError):0
if PY2:
	def get_classobj_bases(cls):
		bases=[cls]
		for base in cls.__bases__:bases.extend(get_classobj_bases(base))
		return bases
class BaseRepresenter:
	yaml_representers={};yaml_multi_representers={}
	def __init__(self,default_style=_A,default_flow_style=_A,dumper=_A):
		self.dumper=dumper
		if self.dumper is not _A:self.dumper._representer=self
		self.default_style=default_style;self.default_flow_style=default_flow_style;self.represented_objects={};self.object_keeper=[];self.alias_key=_A;self.sort_base_mapping_type_on_output=_B
	@property
	def serializer(self):
		try:
			if hasattr(self.dumper,_S):return self.dumper.serializer
			return self.dumper._serializer
		except AttributeError:return self
	def represent(self,data):node=self.represent_data(data);self.serializer.serialize(node);self.represented_objects={};self.object_keeper=[];self.alias_key=_A
	def represent_data(self,data):
		if self.ignore_aliases(data):self.alias_key=_A
		else:self.alias_key=id(data)
		if self.alias_key is not _A:
			if self.alias_key in self.represented_objects:node=self.represented_objects[self.alias_key];return node
			self.object_keeper.append(data)
		data_types=type(data).__mro__
		if PY2:
			if isinstance(data,types.InstanceType):data_types=get_classobj_bases(data.__class__)+list(data_types)
		if data_types[0]in self.yaml_representers:node=self.yaml_representers[data_types[0]](self,data)
		else:
			for data_type in data_types:
				if data_type in self.yaml_multi_representers:node=self.yaml_multi_representers[data_type](self,data);break
			else:
				if _A in self.yaml_multi_representers:node=self.yaml_multi_representers[_A](self,data)
				elif _A in self.yaml_representers:node=self.yaml_representers[_A](self,data)
				else:node=ScalarNode(_A,text_type(data))
		return node
	def represent_key(self,data):return self.represent_data(data)
	@classmethod
	def add_representer(cls,data_type,representer):
		if'yaml_representers'not in cls.__dict__:cls.yaml_representers=cls.yaml_representers.copy()
		cls.yaml_representers[data_type]=representer
	@classmethod
	def add_multi_representer(cls,data_type,representer):
		if'yaml_multi_representers'not in cls.__dict__:cls.yaml_multi_representers=cls.yaml_multi_representers.copy()
		cls.yaml_multi_representers[data_type]=representer
	def represent_scalar(self,tag,value,style=_A,anchor=_A):
		if style is _A:style=self.default_style
		comment=_A
		if style and style[0]in'|>':
			comment=getattr(value,_F,_A)
			if comment:comment=[_A,[comment]]
		node=ScalarNode(tag,value,style=style,comment=comment,anchor=anchor)
		if self.alias_key is not _A:self.represented_objects[self.alias_key]=node
		return node
	def represent_sequence(self,tag,sequence,flow_style=_A):
		value=[];node=SequenceNode(tag,value,flow_style=flow_style)
		if self.alias_key is not _A:self.represented_objects[self.alias_key]=node
		best_style=_B
		for item in sequence:
			node_item=self.represent_data(item)
			if not(isinstance(node_item,ScalarNode)and not node_item.style):best_style=_C
			value.append(node_item)
		if flow_style is _A:
			if self.default_flow_style is not _A:node.flow_style=self.default_flow_style
			else:node.flow_style=best_style
		return node
	def represent_omap(self,tag,omap,flow_style=_A):
		value=[];node=SequenceNode(tag,value,flow_style=flow_style)
		if self.alias_key is not _A:self.represented_objects[self.alias_key]=node
		best_style=_B
		for item_key in omap:item_val=omap[item_key];node_item=self.represent_data({item_key:item_val});value.append(node_item)
		if flow_style is _A:
			if self.default_flow_style is not _A:node.flow_style=self.default_flow_style
			else:node.flow_style=best_style
		return node
	def represent_mapping(self,tag,mapping,flow_style=_A):
		value=[];node=MappingNode(tag,value,flow_style=flow_style)
		if self.alias_key is not _A:self.represented_objects[self.alias_key]=node
		best_style=_B
		if hasattr(mapping,'items'):
			mapping=list(mapping.items())
			if self.sort_base_mapping_type_on_output:
				try:mapping=sorted(mapping)
				except TypeError:pass
		for (item_key,item_value) in mapping:
			node_key=self.represent_key(item_key);node_value=self.represent_data(item_value)
			if not(isinstance(node_key,ScalarNode)and not node_key.style):best_style=_C
			if not(isinstance(node_value,ScalarNode)and not node_value.style):best_style=_C
			value.append((node_key,node_value))
		if flow_style is _A:
			if self.default_flow_style is not _A:node.flow_style=self.default_flow_style
			else:node.flow_style=best_style
		return node
	def ignore_aliases(self,data):return _C
class SafeRepresenter(BaseRepresenter):
	def ignore_aliases(self,data):
		if data is _A or isinstance(data,tuple)and data==():return _B
		if isinstance(data,(binary_type,text_type,bool,int,float)):return _B
		return _C
	def represent_none(self,data):return self.represent_scalar(_L,_T)
	if PY3:
		def represent_str(self,data):return self.represent_scalar(_D,data)
		def represent_binary(self,data):
			if hasattr(base64,'encodebytes'):data=base64.encodebytes(data).decode(_E)
			else:data=base64.encodestring(data).decode(_E)
			return self.represent_scalar(_M,data,style=_J)
	else:
		def represent_str(self,data):
			tag=_A;style=_A
			try:data=unicode(data,_E);tag=_D
			except UnicodeDecodeError:
				try:data=unicode(data,_U);tag=_D
				except UnicodeDecodeError:data=data.encode(_V);tag=_M;style=_J
			return self.represent_scalar(tag,data,style=style)
		def represent_unicode(self,data):return self.represent_scalar(_D,data)
	def represent_bool(self,data,anchor=_A):
		try:value=self.dumper.boolean_representation[bool(data)]
		except AttributeError:
			if data:value='true'
			else:value='false'
		return self.represent_scalar('tag:yaml.org,2002:bool',value,anchor=anchor)
	def represent_int(self,data):return self.represent_scalar(_G,text_type(data))
	if PY2:
		def represent_long(self,data):return self.represent_scalar(_G,text_type(data))
	inf_value=1e+300
	while repr(inf_value)!=repr(inf_value*inf_value):inf_value*=inf_value
	def represent_float(self,data):
		if data!=data or data==_K and data==1.0:value=_W
		elif data==self.inf_value:value=_X
		elif data==-self.inf_value:value=_Y
		else:
			value=to_unicode(repr(data)).lower()
			if getattr(self.serializer,'use_version',_A)==(1,1):
				if _H not in value and'e'in value:value=value.replace('e','.0e',1)
		return self.represent_scalar(_N,value)
	def represent_list(self,data):return self.represent_sequence(_O,data)
	def represent_dict(self,data):return self.represent_mapping(_P,data)
	def represent_ordereddict(self,data):return self.represent_omap('tag:yaml.org,2002:omap',data)
	def represent_set(self,data):
		value={}
		for key in data:value[key]=_A
		return self.represent_mapping(_Z,value)
	def represent_date(self,data):value=to_unicode(data.isoformat());return self.represent_scalar(_Q,value)
	def represent_datetime(self,data):value=to_unicode(data.isoformat(' '));return self.represent_scalar(_Q,value)
	def represent_yaml_object(self,tag,data,cls,flow_style=_A):
		if hasattr(data,_a):state=data.__getstate__()
		else:state=data.__dict__.copy()
		return self.represent_mapping(tag,state,flow_style=flow_style)
	def represent_undefined(self,data):raise RepresenterError('cannot represent an object: %s'%(data,))
SafeRepresenter.add_representer(type(_A),SafeRepresenter.represent_none)
SafeRepresenter.add_representer(str,SafeRepresenter.represent_str)
if PY2:SafeRepresenter.add_representer(unicode,SafeRepresenter.represent_unicode)
else:SafeRepresenter.add_representer(bytes,SafeRepresenter.represent_binary)
SafeRepresenter.add_representer(bool,SafeRepresenter.represent_bool)
SafeRepresenter.add_representer(int,SafeRepresenter.represent_int)
if PY2:SafeRepresenter.add_representer(long,SafeRepresenter.represent_long)
SafeRepresenter.add_representer(float,SafeRepresenter.represent_float)
SafeRepresenter.add_representer(list,SafeRepresenter.represent_list)
SafeRepresenter.add_representer(tuple,SafeRepresenter.represent_list)
SafeRepresenter.add_representer(dict,SafeRepresenter.represent_dict)
SafeRepresenter.add_representer(set,SafeRepresenter.represent_set)
SafeRepresenter.add_representer(ordereddict,SafeRepresenter.represent_ordereddict)
if sys.version_info>=(2,7):import collections;SafeRepresenter.add_representer(collections.OrderedDict,SafeRepresenter.represent_ordereddict)
SafeRepresenter.add_representer(datetime.date,SafeRepresenter.represent_date)
SafeRepresenter.add_representer(datetime.datetime,SafeRepresenter.represent_datetime)
SafeRepresenter.add_representer(_A,SafeRepresenter.represent_undefined)
class Representer(SafeRepresenter):
	if PY2:
		def represent_str(self,data):
			tag=_A;style=_A
			try:data=unicode(data,_E);tag=_D
			except UnicodeDecodeError:
				try:data=unicode(data,_U);tag='tag:yaml.org,2002:python/str'
				except UnicodeDecodeError:data=data.encode(_V);tag=_M;style=_J
			return self.represent_scalar(tag,data,style=style)
		def represent_unicode(self,data):
			tag=_A
			try:data.encode(_E);tag='tag:yaml.org,2002:python/unicode'
			except UnicodeEncodeError:tag=_D
			return self.represent_scalar(tag,data)
		def represent_long(self,data):
			tag=_G
			if int(data)is not data:tag='tag:yaml.org,2002:python/long'
			return self.represent_scalar(tag,to_unicode(data))
	def represent_complex(self,data):
		if data.imag==_K:data='%r'%data.real
		elif data.real==_K:data='%rj'%data.imag
		elif data.imag>0:data='%r+%rj'%(data.real,data.imag)
		else:data='%r%rj'%(data.real,data.imag)
		return self.represent_scalar('tag:yaml.org,2002:python/complex',data)
	def represent_tuple(self,data):return self.represent_sequence('tag:yaml.org,2002:python/tuple',data)
	def represent_name(self,data):
		try:name=_I%(data.__module__,data.__qualname__)
		except AttributeError:name=_I%(data.__module__,data.__name__)
		return self.represent_scalar('tag:yaml.org,2002:python/name:'+name,'')
	def represent_module(self,data):return self.represent_scalar('tag:yaml.org,2002:python/module:'+data.__name__,'')
	if PY2:
		def represent_instance(self,data):
			cls=data.__class__;class_name=_I%(cls.__module__,cls.__name__);args=_A;state=_A
			if hasattr(data,'__getinitargs__'):args=list(data.__getinitargs__())
			if hasattr(data,_a):state=data.__getstate__()
			else:state=data.__dict__
			if args is _A and isinstance(state,dict):return self.represent_mapping(_b+class_name,state)
			if isinstance(state,dict)and not state:return self.represent_sequence(_R+class_name,args)
			value={}
			if bool(args):value[_c]=args
			value[_d]=state;return self.represent_mapping(_R+class_name,value)
	def represent_object(self,data):
		cls=type(data)
		if cls in copyreg.dispatch_table:reduce=copyreg.dispatch_table[cls](data)
		elif hasattr(data,'__reduce_ex__'):reduce=data.__reduce_ex__(2)
		elif hasattr(data,'__reduce__'):reduce=data.__reduce__()
		else:raise RepresenterError('cannot represent object: %r'%(data,))
		reduce=(list(reduce)+[_A]*5)[:5];function,args,state,listitems,dictitems=reduce;args=list(args)
		if state is _A:state={}
		if listitems is not _A:listitems=list(listitems)
		if dictitems is not _A:dictitems=dict(dictitems)
		if function.__name__=='__newobj__':function=args[0];args=args[1:];tag=_R;newobj=_B
		else:tag='tag:yaml.org,2002:python/object/apply:';newobj=_C
		try:function_name=_I%(function.__module__,function.__qualname__)
		except AttributeError:function_name=_I%(function.__module__,function.__name__)
		if not args and not listitems and not dictitems and isinstance(state,dict)and newobj:return self.represent_mapping(_b+function_name,state)
		if not listitems and not dictitems and isinstance(state,dict)and not state:return self.represent_sequence(tag+function_name,args)
		value={}
		if args:value[_c]=args
		if state or not isinstance(state,dict):value[_d]=state
		if listitems:value['listitems']=listitems
		if dictitems:value['dictitems']=dictitems
		return self.represent_mapping(tag+function_name,value)
if PY2:Representer.add_representer(str,Representer.represent_str);Representer.add_representer(unicode,Representer.represent_unicode);Representer.add_representer(long,Representer.represent_long)
Representer.add_representer(complex,Representer.represent_complex)
Representer.add_representer(tuple,Representer.represent_tuple)
Representer.add_representer(type,Representer.represent_name)
if PY2:Representer.add_representer(types.ClassType,Representer.represent_name)
Representer.add_representer(types.FunctionType,Representer.represent_name)
Representer.add_representer(types.BuiltinFunctionType,Representer.represent_name)
Representer.add_representer(types.ModuleType,Representer.represent_module)
if PY2:Representer.add_multi_representer(types.InstanceType,Representer.represent_instance)
Representer.add_multi_representer(object,Representer.represent_object)
Representer.add_multi_representer(type,Representer.represent_name)
from .comments import CommentedMap,CommentedOrderedMap,CommentedSeq,CommentedKeySeq,CommentedKeyMap,CommentedSet,comment_attrib,merge_attrib,TaggedScalar
class RoundTripRepresenter(SafeRepresenter):
	def __init__(self,default_style=_A,default_flow_style=_A,dumper=_A):
		if not hasattr(dumper,_S)and default_flow_style is _A:default_flow_style=_C
		SafeRepresenter.__init__(self,default_style=default_style,default_flow_style=default_flow_style,dumper=dumper)
	def ignore_aliases(self,data):
		try:
			if data.anchor is not _A and data.anchor.value is not _A:return _C
		except AttributeError:pass
		return SafeRepresenter.ignore_aliases(self,data)
	def represent_none(self,data):
		if len(self.represented_objects)==0 and not self.serializer.use_explicit_start:return self.represent_scalar(_L,_T)
		return self.represent_scalar(_L,'')
	def represent_literal_scalarstring(self,data):
		tag=_A;style=_J;anchor=data.yaml_anchor(any=_B)
		if PY2 and not isinstance(data,unicode):data=unicode(data,_E)
		tag=_D;return self.represent_scalar(tag,data,style=style,anchor=anchor)
	represent_preserved_scalarstring=represent_literal_scalarstring
	def represent_folded_scalarstring(self,data):
		tag=_A;style='>';anchor=data.yaml_anchor(any=_B)
		for fold_pos in reversed(getattr(data,'fold_pos',[])):
			if data[fold_pos]==' 'and(fold_pos>0 and not data[fold_pos-1].isspace())and(fold_pos<len(data)and not data[fold_pos+1].isspace()):data=data[:fold_pos]+'\x07'+data[fold_pos:]
		if PY2 and not isinstance(data,unicode):data=unicode(data,_E)
		tag=_D;return self.represent_scalar(tag,data,style=style,anchor=anchor)
	def represent_single_quoted_scalarstring(self,data):
		tag=_A;style="'";anchor=data.yaml_anchor(any=_B)
		if PY2 and not isinstance(data,unicode):data=unicode(data,_E)
		tag=_D;return self.represent_scalar(tag,data,style=style,anchor=anchor)
	def represent_double_quoted_scalarstring(self,data):
		tag=_A;style='"';anchor=data.yaml_anchor(any=_B)
		if PY2 and not isinstance(data,unicode):data=unicode(data,_E)
		tag=_D;return self.represent_scalar(tag,data,style=style,anchor=anchor)
	def represent_plain_scalarstring(self,data):
		tag=_A;style='';anchor=data.yaml_anchor(any=_B)
		if PY2 and not isinstance(data,unicode):data=unicode(data,_E)
		tag=_D;return self.represent_scalar(tag,data,style=style,anchor=anchor)
	def insert_underscore(self,prefix,s,underscore,anchor=_A):
		A='_'
		if underscore is _A:return self.represent_scalar(_G,prefix+s,anchor=anchor)
		if underscore[0]:
			sl=list(s);pos=len(s)-underscore[0]
			while pos>0:sl.insert(pos,A);pos-=underscore[0]
			s=''.join(sl)
		if underscore[1]:s=A+s
		if underscore[2]:s+=A
		return self.represent_scalar(_G,prefix+s,anchor=anchor)
	def represent_scalar_int(self,data):
		if data._width is not _A:s='{:0{}d}'.format(data,data._width)
		else:s=format(data,'d')
		anchor=data.yaml_anchor(any=_B);return self.insert_underscore('',s,data._underscore,anchor=anchor)
	def represent_binary_int(self,data):
		if data._width is not _A:s='{:0{}b}'.format(data,data._width)
		else:s=format(data,'b')
		anchor=data.yaml_anchor(any=_B);return self.insert_underscore('0b',s,data._underscore,anchor=anchor)
	def represent_octal_int(self,data):
		if data._width is not _A:s='{:0{}o}'.format(data,data._width)
		else:s=format(data,'o')
		anchor=data.yaml_anchor(any=_B);return self.insert_underscore('0o',s,data._underscore,anchor=anchor)
	def represent_hex_int(self,data):
		if data._width is not _A:s='{:0{}x}'.format(data,data._width)
		else:s=format(data,'x')
		anchor=data.yaml_anchor(any=_B);return self.insert_underscore('0x',s,data._underscore,anchor=anchor)
	def represent_hex_caps_int(self,data):
		if data._width is not _A:s='{:0{}X}'.format(data,data._width)
		else:s=format(data,'X')
		anchor=data.yaml_anchor(any=_B);return self.insert_underscore('0x',s,data._underscore,anchor=anchor)
	def represent_scalar_float(self,data):
		C='+';B='{:{}0{}d}';A='0';value=_A;anchor=data.yaml_anchor(any=_B)
		if data!=data or data==_K and data==1.0:value=_W
		elif data==self.inf_value:value=_X
		elif data==-self.inf_value:value=_Y
		if value:return self.represent_scalar(_N,value,anchor=anchor)
		if data._exp is _A and data._prec>0 and data._prec==data._width-1:value='{}{:d}.'.format(data._m_sign if data._m_sign else'',abs(int(data)))
		elif data._exp is _A:
			prec=data._prec;ms=data._m_sign if data._m_sign else'';value='{}{:0{}.{}f}'.format(ms,abs(data),data._width-len(ms),data._width-prec-1)
			if prec==0 or prec==1 and ms!='':value=value.replace('0.',_H)
			while len(value)<data._width:value+=A
		else:
			m,es='{:{}.{}e}'.format(data,data._width,data._width+(1 if data._m_sign else 0)).split('e');w=data._width if data._prec>0 else data._width+1
			if data<0:w+=1
			m=m[:w];e=int(es);m1,m2=m.split(_H)
			while len(m1)+len(m2)<data._width-(1 if data._prec>=0 else 0):m2+=A
			if data._m_sign and data>0:m1=C+m1
			esgn=C if data._e_sign else''
			if data._prec<0:
				if m2!=A:e-=len(m2)
				else:m2=''
				while len(m1)+len(m2)-(1 if data._m_sign else 0)<data._width:m2+=A;e-=1
				value=m1+m2+data._exp+B.format(e,esgn,data._e_width)
			elif data._prec==0:e-=len(m2);value=m1+m2+_H+data._exp+B.format(e,esgn,data._e_width)
			else:
				if data._m_lead0>0:m2=A*(data._m_lead0-1)+m1+m2;m1=A;m2=m2[:-data._m_lead0];e+=data._m_lead0
				while len(m1)<data._prec:m1+=m2[0];m2=m2[1:];e-=1
				value=m1+_H+m2+data._exp+B.format(e,esgn,data._e_width)
		if value is _A:value=to_unicode(repr(data)).lower()
		return self.represent_scalar(_N,value,anchor=anchor)
	def represent_sequence(self,tag,sequence,flow_style=_A):
		value=[]
		try:flow_style=sequence.fa.flow_style(flow_style)
		except AttributeError:flow_style=flow_style
		try:anchor=sequence.yaml_anchor()
		except AttributeError:anchor=_A
		node=SequenceNode(tag,value,flow_style=flow_style,anchor=anchor)
		if self.alias_key is not _A:self.represented_objects[self.alias_key]=node
		best_style=_B
		try:
			comment=getattr(sequence,comment_attrib);node.comment=comment.comment
			if node.comment and node.comment[1]:
				for ct in node.comment[1]:ct.reset()
			item_comments=comment.items
			for v in item_comments.values():
				if v and v[1]:
					for ct in v[1]:ct.reset()
			item_comments=comment.items;node.comment=comment.comment
			try:node.comment.append(comment.end)
			except AttributeError:pass
		except AttributeError:item_comments={}
		for (idx,item) in enumerate(sequence):
			node_item=self.represent_data(item);self.merge_comments(node_item,item_comments.get(idx))
			if not(isinstance(node_item,ScalarNode)and not node_item.style):best_style=_C
			value.append(node_item)
		if flow_style is _A:
			if len(sequence)!=0 and self.default_flow_style is not _A:node.flow_style=self.default_flow_style
			else:node.flow_style=best_style
		return node
	def merge_comments(self,node,comments):
		if comments is _A:assert hasattr(node,_F);return node
		if getattr(node,_F,_A)is not _A:
			for (idx,val) in enumerate(comments):
				if idx>=len(node.comment):continue
				nc=node.comment[idx]
				if nc is not _A:assert val is _A or val==nc;comments[idx]=nc
		node.comment=comments;return node
	def represent_key(self,data):
		if isinstance(data,CommentedKeySeq):self.alias_key=_A;return self.represent_sequence(_O,data,flow_style=_B)
		if isinstance(data,CommentedKeyMap):self.alias_key=_A;return self.represent_mapping(_P,data,flow_style=_B)
		return SafeRepresenter.represent_key(self,data)
	def represent_mapping(self,tag,mapping,flow_style=_A):
		value=[]
		try:flow_style=mapping.fa.flow_style(flow_style)
		except AttributeError:flow_style=flow_style
		try:anchor=mapping.yaml_anchor()
		except AttributeError:anchor=_A
		node=MappingNode(tag,value,flow_style=flow_style,anchor=anchor)
		if self.alias_key is not _A:self.represented_objects[self.alias_key]=node
		best_style=_B
		try:
			comment=getattr(mapping,comment_attrib);node.comment=comment.comment
			if node.comment and node.comment[1]:
				for ct in node.comment[1]:ct.reset()
			item_comments=comment.items
			for v in item_comments.values():
				if v and v[1]:
					for ct in v[1]:ct.reset()
			try:node.comment.append(comment.end)
			except AttributeError:pass
		except AttributeError:item_comments={}
		merge_list=[m[1]for m in getattr(mapping,merge_attrib,[])]
		try:merge_pos=getattr(mapping,merge_attrib,[[0]])[0][0]
		except IndexError:merge_pos=0
		item_count=0
		if bool(merge_list):items=mapping.non_merged_items()
		else:items=mapping.items()
		for (item_key,item_value) in items:
			item_count+=1;node_key=self.represent_key(item_key);node_value=self.represent_data(item_value);item_comment=item_comments.get(item_key)
			if item_comment:
				assert getattr(node_key,_F,_A)is _A;node_key.comment=item_comment[:2];nvc=getattr(node_value,_F,_A)
				if nvc is not _A:nvc[0]=item_comment[2];nvc[1]=item_comment[3]
				else:node_value.comment=item_comment[2:]
			if not(isinstance(node_key,ScalarNode)and not node_key.style):best_style=_C
			if not(isinstance(node_value,ScalarNode)and not node_value.style):best_style=_C
			value.append((node_key,node_value))
		if flow_style is _A:
			if(item_count!=0 or bool(merge_list))and self.default_flow_style is not _A:node.flow_style=self.default_flow_style
			else:node.flow_style=best_style
		if bool(merge_list):
			if len(merge_list)==1:arg=self.represent_data(merge_list[0])
			else:arg=self.represent_data(merge_list);arg.flow_style=_B
			value.insert(merge_pos,(ScalarNode('tag:yaml.org,2002:merge','<<'),arg))
		return node
	def represent_omap(self,tag,omap,flow_style=_A):
		value=[]
		try:flow_style=omap.fa.flow_style(flow_style)
		except AttributeError:flow_style=flow_style
		try:anchor=omap.yaml_anchor()
		except AttributeError:anchor=_A
		node=SequenceNode(tag,value,flow_style=flow_style,anchor=anchor)
		if self.alias_key is not _A:self.represented_objects[self.alias_key]=node
		best_style=_B
		try:
			comment=getattr(omap,comment_attrib);node.comment=comment.comment
			if node.comment and node.comment[1]:
				for ct in node.comment[1]:ct.reset()
			item_comments=comment.items
			for v in item_comments.values():
				if v and v[1]:
					for ct in v[1]:ct.reset()
			try:node.comment.append(comment.end)
			except AttributeError:pass
		except AttributeError:item_comments={}
		for item_key in omap:
			item_val=omap[item_key];node_item=self.represent_data({item_key:item_val});item_comment=item_comments.get(item_key)
			if item_comment:
				if item_comment[1]:node_item.comment=[_A,item_comment[1]]
				assert getattr(node_item.value[0][0],_F,_A)is _A;node_item.value[0][0].comment=[item_comment[0],_A];nvc=getattr(node_item.value[0][1],_F,_A)
				if nvc is not _A:nvc[0]=item_comment[2];nvc[1]=item_comment[3]
				else:node_item.value[0][1].comment=item_comment[2:]
			value.append(node_item)
		if flow_style is _A:
			if self.default_flow_style is not _A:node.flow_style=self.default_flow_style
			else:node.flow_style=best_style
		return node
	def represent_set(self,setting):
		flow_style=_C;tag=_Z;value=[];flow_style=setting.fa.flow_style(flow_style)
		try:anchor=setting.yaml_anchor()
		except AttributeError:anchor=_A
		node=MappingNode(tag,value,flow_style=flow_style,anchor=anchor)
		if self.alias_key is not _A:self.represented_objects[self.alias_key]=node
		best_style=_B
		try:
			comment=getattr(setting,comment_attrib);node.comment=comment.comment
			if node.comment and node.comment[1]:
				for ct in node.comment[1]:ct.reset()
			item_comments=comment.items
			for v in item_comments.values():
				if v and v[1]:
					for ct in v[1]:ct.reset()
			try:node.comment.append(comment.end)
			except AttributeError:pass
		except AttributeError:item_comments={}
		for item_key in setting.odict:
			node_key=self.represent_key(item_key);node_value=self.represent_data(_A);item_comment=item_comments.get(item_key)
			if item_comment:assert getattr(node_key,_F,_A)is _A;node_key.comment=item_comment[:2]
			node_key.style=node_value.style='?'
			if not(isinstance(node_key,ScalarNode)and not node_key.style):best_style=_C
			if not(isinstance(node_value,ScalarNode)and not node_value.style):best_style=_C
			value.append((node_key,node_value))
		best_style=best_style;return node
	def represent_dict(self,data):
		try:t=data.tag.value
		except AttributeError:t=_A
		if t:
			if t.startswith('!!'):tag=_e+t[2:]
			else:tag=t
		else:tag=_P
		return self.represent_mapping(tag,data)
	def represent_list(self,data):
		try:t=data.tag.value
		except AttributeError:t=_A
		if t:
			if t.startswith('!!'):tag=_e+t[2:]
			else:tag=t
		else:tag=_O
		return self.represent_sequence(tag,data)
	def represent_datetime(self,data):
		B='tz';A='delta';inter='T'if data._yaml['t']else' ';_yaml=data._yaml
		if _yaml[A]:data+=_yaml[A];value=data.isoformat(inter)
		else:value=data.isoformat(inter)
		if _yaml[B]:value+=_yaml[B]
		return self.represent_scalar(_Q,to_unicode(value))
	def represent_tagged_scalar(self,data):
		try:tag=data.tag.value
		except AttributeError:tag=_A
		try:anchor=data.yaml_anchor()
		except AttributeError:anchor=_A
		return self.represent_scalar(tag,data.value,style=data.style,anchor=anchor)
	def represent_scalar_bool(self,data):
		try:anchor=data.yaml_anchor()
		except AttributeError:anchor=_A
		return SafeRepresenter.represent_bool(self,data,anchor=anchor)
RoundTripRepresenter.add_representer(type(_A),RoundTripRepresenter.represent_none)
RoundTripRepresenter.add_representer(LiteralScalarString,RoundTripRepresenter.represent_literal_scalarstring)
RoundTripRepresenter.add_representer(FoldedScalarString,RoundTripRepresenter.represent_folded_scalarstring)
RoundTripRepresenter.add_representer(SingleQuotedScalarString,RoundTripRepresenter.represent_single_quoted_scalarstring)
RoundTripRepresenter.add_representer(DoubleQuotedScalarString,RoundTripRepresenter.represent_double_quoted_scalarstring)
RoundTripRepresenter.add_representer(PlainScalarString,RoundTripRepresenter.represent_plain_scalarstring)
RoundTripRepresenter.add_representer(ScalarInt,RoundTripRepresenter.represent_scalar_int)
RoundTripRepresenter.add_representer(BinaryInt,RoundTripRepresenter.represent_binary_int)
RoundTripRepresenter.add_representer(OctalInt,RoundTripRepresenter.represent_octal_int)
RoundTripRepresenter.add_representer(HexInt,RoundTripRepresenter.represent_hex_int)
RoundTripRepresenter.add_representer(HexCapsInt,RoundTripRepresenter.represent_hex_caps_int)
RoundTripRepresenter.add_representer(ScalarFloat,RoundTripRepresenter.represent_scalar_float)
RoundTripRepresenter.add_representer(ScalarBoolean,RoundTripRepresenter.represent_scalar_bool)
RoundTripRepresenter.add_representer(CommentedSeq,RoundTripRepresenter.represent_list)
RoundTripRepresenter.add_representer(CommentedMap,RoundTripRepresenter.represent_dict)
RoundTripRepresenter.add_representer(CommentedOrderedMap,RoundTripRepresenter.represent_ordereddict)
if sys.version_info>=(2,7):import collections;RoundTripRepresenter.add_representer(collections.OrderedDict,RoundTripRepresenter.represent_ordereddict)
RoundTripRepresenter.add_representer(CommentedSet,RoundTripRepresenter.represent_set)
RoundTripRepresenter.add_representer(TaggedScalar,RoundTripRepresenter.represent_tagged_scalar)
RoundTripRepresenter.add_representer(TimeStamp,RoundTripRepresenter.represent_datetime)


from json import JSONDecodeError
from pathlib import Path
from typing import Union
from dynaconf.vendor.toml import TomlDecodeError
from dynaconf.vendor.ruamel.yaml import YAMLError
from .exceptions import BoxError
from .box import Box
from .box_list import BoxList
__all__=['box_from_file']
def _to_json(data):
	try:return Box.from_json(data)
	except JSONDecodeError:raise BoxError('File is not JSON as expected')
	except BoxError:return BoxList.from_json(data)
def _to_yaml(data):
	try:return Box.from_yaml(data)
	except YAMLError:raise BoxError('File is not YAML as expected')
	except BoxError:return BoxList.from_yaml(data)
def _to_toml(data):
	try:return Box.from_toml(data)
	except TomlDecodeError:raise BoxError('File is not TOML as expected')
def box_from_file(file,file_type=None,encoding='utf-8',errors='strict'):
	C=file_type;A=file
	if not isinstance(A,Path):A=Path(A)
	if not A.exists():raise BoxError(f'file "{A}" does not exist')
	B=A.read_text(encoding=encoding,errors=errors)
	if C:
		if C.lower()=='json':return _to_json(B)
		if C.lower()=='yaml':return _to_yaml(B)
		if C.lower()=='toml':return _to_toml(B)
		raise BoxError(f'"{C}" is an unknown type, please use either toml, yaml or json')
	if A.suffix in('.json','.jsn'):return _to_json(B)
	if A.suffix in('.yaml','.yml'):return _to_yaml(B)
	if A.suffix in('.tml','.toml'):return _to_toml(B)
	raise BoxError(f"Could not determine file type based off extension, please provide file_type")


import codecs,os
def _verify_python_env():
	M='.utf8';L='.utf-8';J=None;I='ascii'
	try:import locale as A;G=codecs.lookup(A.getpreferredencoding()).name
	except Exception:G=I
	if G!=I:return
	B=''
	if os.name=='posix':
		import subprocess as D
		try:C=D.Popen(['locale','-a'],stdout=D.PIPE,stderr=D.PIPE).communicate()[0]
		except OSError:C=b''
		E=set();H=False
		if isinstance(C,bytes):C=C.decode(I,'replace')
		for K in C.splitlines():
			A=K.strip()
			if A.lower().endswith((L,M)):
				E.add(A)
				if A.lower()in('c.utf8','c.utf-8'):H=True
		B+='\n\n'
		if not E:B+='Additional information: on this system no suitable UTF-8 locales were discovered. This most likely requires resolving by reconfiguring the locale system.'
		elif H:B+='This system supports the C.UTF-8 locale which is recommended. You might be able to resolve your issue by exporting the following environment variables:\n\n    export LC_ALL=C.UTF-8\n    export LANG=C.UTF-8'
		else:B+=f"This system lists some UTF-8 supporting locales that you can pick from. The following suitable locales were discovered: {', '.join(sorted(E))}"
		F=J
		for A in (os.environ.get('LC_ALL'),os.environ.get('LANG')):
			if A and A.lower().endswith((L,M)):F=A
			if A is not J:break
		if F is not J:B+=f"\n\nClick discovered that you exported a UTF-8 locale but the locale system could not pick up from it because it does not exist. The exported locale is {F!r} but it is not supported"
	raise RuntimeError(f"Click will abort further execution because Python was configured to use ASCII as encoding for the environment. Consult https://click.palletsprojects.com/unicode-support/ for mitigation steps.{B}")


__author__='Chris Griffith'
__version__='4.2.3'
from .box import Box
from .box_list import BoxList
from .config_box import ConfigBox
from .shorthand_box import SBox
from .exceptions import BoxError,BoxKeyError
from .from_file import box_from_file


from __future__ import print_function
_A='store_true'
from IPython.core.magic import Magics,line_magic,magics_class
from IPython.core.magic_arguments import argument,magic_arguments,parse_argstring
from .main import find_dotenv,load_dotenv
@magics_class
class IPythonDotEnv(Magics):
	@magic_arguments()
	@argument('-o','--override',action=_A,help='Indicate to override existing variables')
	@argument('-v','--verbose',action=_A,help='Indicate function calls to be verbose')
	@argument('dotenv_path',nargs='?',type=str,default='.env',help='Search in increasingly higher folders for the `dotenv_path`')
	@line_magic
	def dotenv(self,line):
		C=True;A=parse_argstring(self.dotenv,line);B=A.dotenv_path
		try:B=find_dotenv(B,C,C)
		except IOError:print('cannot find .env file');return
		load_dotenv(B,verbose=A.verbose,override=A.override)
def load_ipython_extension(ipython):ipython.register_magics(IPythonDotEnv)


_C=True
_B=False
_A=None
import inspect,io,itertools,os,struct,sys
from ._compat import DEFAULT_COLUMNS,get_winterm_size,isatty,strip_ansi,WIN
from .exceptions import Abort
from .exceptions import UsageError
from .globals import resolve_color_default
from .types import Choice
from .types import convert_type
from .types import Path
from .utils import echo
from .utils import LazyFile
visible_prompt_func=input
_ansi_colors={'black':30,'red':31,'green':32,'yellow':33,'blue':34,'magenta':35,'cyan':36,'white':37,'reset':39,'bright_black':90,'bright_red':91,'bright_green':92,'bright_yellow':93,'bright_blue':94,'bright_magenta':95,'bright_cyan':96,'bright_white':97}
_ansi_reset_all='\x1b[0m'
def hidden_prompt_func(prompt):import getpass as A;return A.getpass(prompt)
def _build_prompt(text,suffix,show_default=_B,default=_A,show_choices=_C,type=_A):
	B=default;A=text
	if type is not _A and show_choices and isinstance(type,Choice):A+=f" ({', '.join(map(str,type.choices))})"
	if B is not _A and show_default:A=f"{A} [{_format_default(B)}]"
	return f"{A}{suffix}"
def _format_default(default):
	A=default
	if isinstance(A,(io.IOBase,LazyFile))and hasattr(A,'name'):return A.name
	return A
def prompt(text,default=_A,hide_input=_B,confirmation_prompt=_B,type=_A,value_proc=_A,prompt_suffix=': ',show_default=_C,err=_B,show_choices=_C):
	F=hide_input;C=err;B=value_proc;A=default;E=_A
	def G(text):
		A=hidden_prompt_func if F else visible_prompt_func
		try:echo(text,nl=_B,err=C);return A('')
		except (KeyboardInterrupt,EOFError):
			if F:echo(_A,err=C)
			raise Abort()
	if B is _A:B=convert_type(type,A)
	I=_build_prompt(text,prompt_suffix,show_default,A,show_choices,type)
	while 1:
		while 1:
			D=G(I)
			if D:break
			elif A is not _A:
				if isinstance(B,Path):D=A;break
				return A
		try:E=B(D)
		except UsageError as J:echo(f"Error: {J.message}",err=C);continue
		if not confirmation_prompt:return E
		while 1:
			H=G('Repeat for confirmation: ')
			if H:break
		if D==H:return E
		echo('Error: the two entered values do not match',err=C)
def confirm(text,default=_B,abort=_B,prompt_suffix=': ',show_default=_C,err=_B):
	C=default;D=_build_prompt(text,prompt_suffix,show_default,'Y/n'if C else'y/N')
	while 1:
		try:echo(D,nl=_B,err=err);B=visible_prompt_func('').lower().strip()
		except (KeyboardInterrupt,EOFError):raise Abort()
		if B in('y','yes'):A=_C
		elif B in('n','no'):A=_B
		elif B=='':A=C
		else:echo('Error: invalid input',err=err);continue
		break
	if abort and not A:raise Abort()
	return A
def get_terminal_size():
	import shutil as C
	if hasattr(C,'get_terminal_size'):return C.get_terminal_size()
	if get_winterm_size is not _A:
		D=get_winterm_size()
		if D==(0,0):return 79,24
		else:return D
	def B(fd):
		try:import fcntl,termios as A;B=struct.unpack('hh',fcntl.ioctl(fd,A.TIOCGWINSZ,'1234'))
		except Exception:return
		return B
	A=B(0)or B(1)or B(2)
	if not A:
		try:
			E=os.open(os.ctermid(),os.O_RDONLY)
			try:A=B(E)
			finally:os.close(E)
		except Exception:pass
	if not A or not A[0]or not A[1]:A=os.environ.get('LINES',25),os.environ.get('COLUMNS',DEFAULT_COLUMNS)
	return int(A[1]),int(A[0])
def echo_via_pager(text_or_generator,color=_A):
	B=color;A=text_or_generator;B=resolve_color_default(B)
	if inspect.isgeneratorfunction(A):C=A()
	elif isinstance(A,str):C=[A]
	else:C=iter(A)
	D=(A if isinstance(A,str)else str(A)for A in C);from ._termui_impl import pager;return pager(itertools.chain(D,'\n'),B)
def progressbar(iterable=_A,length=_A,label=_A,show_eta=_C,show_percent=_A,show_pos=_B,item_show_func=_A,fill_char='#',empty_char='-',bar_template='%(label)s  [%(bar)s]  %(info)s',info_sep='  ',width=36,file=_A,color=_A):A=color;from ._termui_impl import ProgressBar as B;A=resolve_color_default(A);return B(iterable=iterable,length=length,show_eta=show_eta,show_percent=show_percent,show_pos=show_pos,item_show_func=item_show_func,fill_char=fill_char,empty_char=empty_char,bar_template=bar_template,info_sep=info_sep,file=file,label=label,width=width,color=A)
def clear():
	if not isatty(sys.stdout):return
	if WIN:os.system('cls')
	else:sys.stdout.write('\x1b[2J\x1b[1;1H')
def style(text,fg=_A,bg=_A,bold=_A,dim=_A,underline=_A,blink=_A,reverse=_A,reset=_C):
	D=reverse;C=blink;B=underline;A=[]
	if fg:
		try:A.append(f"[{_ansi_colors[fg]}m")
		except KeyError:raise TypeError(f"Unknown color {fg!r}")
	if bg:
		try:A.append(f"[{_ansi_colors[bg]+10}m")
		except KeyError:raise TypeError(f"Unknown color {bg!r}")
	if bold is not _A:A.append(f"[{1 if bold else 22}m")
	if dim is not _A:A.append(f"[{2 if dim else 22}m")
	if B is not _A:A.append(f"[{4 if B else 24}m")
	if C is not _A:A.append(f"[{5 if C else 25}m")
	if D is not _A:A.append(f"[{7 if D else 27}m")
	A.append(text)
	if reset:A.append(_ansi_reset_all)
	return ''.join(A)
def unstyle(text):return strip_ansi(text)
def secho(message=_A,file=_A,nl=_C,err=_B,color=_A,**B):
	A=message
	if A is not _A:A=style(A,**B)
	return echo(A,file=file,nl=nl,err=err,color=color)
def edit(text=_A,editor=_A,env=_A,require_save=_C,extension='.txt',filename=_A):
	B=filename;A=editor;from ._termui_impl import Editor as C;A=C(editor=A,env=env,require_save=require_save,extension=extension)
	if B is _A:return A.edit(text)
	A.edit_file(B)
def launch(url,wait=_B,locate=_B):from ._termui_impl import open_url as A;return A(url,wait=wait,locate=locate)
_getchar=_A
def getchar(echo=_B):
	A=_getchar
	if A is _A:from ._termui_impl import getchar as A
	return A(echo)
def raw_terminal():from ._termui_impl import raw_terminal as A;return A()
def pause(info='Press any key to continue ...',err=_B):
	A=info
	if not isatty(sys.stdin)or not isatty(sys.stdout):return
	try:
		if A:echo(A,nl=_B,err=err)
		try:getchar()
		except (KeyboardInterrupt,EOFError):pass
	finally:
		if A:echo(err=err)